#!/bin/bash
clear

usage() {
echo "Usage: $0 [OPTIONS]" 1>&2;
echo "This utility used to take backup from CSPC 2.8.1.8/2.9/2.9.1 and import in 2.10"
echo -e "\t -h, --help \t\t\t\t\t   Displays the help"
echo -e "\t -b, --backup \t\t\t\t\t  Take backup of OS/Application files from current box"
echo -e "\t -r, --restore \t\t\t\t\t  Restore OS/Application files from backup"
exit 1;
}

backup() {
uid=`id -u`
if [ "$uid" != "0" ]
then
	echo "This needs to be run from root user"
fi

echo "Enter backup dir , if no input is given then default is /opt "
read INPUT_DIR
if [ -z $INPUT_DIR ]; then
	INPUT_DIR="/opt"
fi

if [ -d $INPUT_DIR/cspc28backup ]; then
	echo -e "\nINFO: Already $INPUT_DIR/cspc28backup exist , cleaning up..."
	rm -rf $INPUT_DIR/cspc28backup
        echo -e "\nINFO: Creating backup dir  $INPUT_DIR/cspc28backup"
        mkdir -p $INPUT_DIR/cspc28backup/backup28
	BACKUP_DIR="$INPUT_DIR/cspc28backup/backup28"

else
	echo -e "\nINFO: Creating backup dir  $INPUT_DIR/cspc28backup"
	mkdir -p $INPUT_DIR/cspc28backup/backup28
	BACKUP_DIR="$INPUT_DIR/cspc28backup/backup28"
fi
if [ -f BackupRestore.sh ]; then
	cp BackupRestore.sh  $INPUT_DIR/cspc28backup

fi

if [ -f /etc/resolv.conf ]; then
	cp /etc/resolv.conf $BACKUP_DIR
	echo -e "INFO: Copied /etc/resolv.conf to backup dir"
fi

if [ -f /etc/logrotate.conf ]; then
        cp /etc/logrotate.conf $BACKUP_DIR
        echo -e "INFO: Copied /etc/logrotate.conf to backup dir"
fi

echo "INFO: Started iptables backup"
iptables-save > $BACKUP_DIR/rules.iptables
echo "INFO: iptables backup completed"


if [ -f /var/spool/cron/root ]; then
	cp /var/spool/cron/root $BACKUP_DIR
	echo "#Ansible: Run AIDE every day" >> $BACKUP_DIR/root
	echo "42 21 * * * nice -n 10 /usr/sbin/aide --update -c /etc/aide.conf -B database=file:/var/lib/aide/aide.db.gz -B database_out=file:/var/lib/aide/aide.db.new.gz && mv /var/lib/aide/aide.db.new.gz /var/lib/aide/aide.db" >> $BACKUP_DIR/root

	echo -e "INFO: Copied /var/spool/cron/root to backup dir"
fi
ls -l /var/spool/cron/ | grep -v "total" | grep -v "root" | awk '{print $9}' > /tmp/cronlist.txt
if [ -f /tmp/cronlist.txt ]; then
     while read line; do
     grep -qzw "/var/spool/cron/$line" files.conf
     file_exist=$?
     if [ -f /var/spool/cron/$line ] && [ $file_exist -ne 0 ]; then
             	echo "/var/spool/cron/$line" >> files.conf
     fi
     done < /tmp/cronlist.txt
fi


if [ -f /etc/hosts ]; then
        cp /etc/hosts $BACKUP_DIR
        echo -e "INFO: Copied /etc/hosts to backup dir"
fi

if [ -f $CSPCHOME/webui/tomcat/conf/cspcgxt ]; then
	cp $CSPCHOME/webui/tomcat/conf/cspcgxt $BACKUP_DIR
	echo "INFO: Copied $CSPCHOME/webui/tomcat/conf/cspcgxt to backup dir"
fi

if [ -f $CSPCHOME/resources/server/mainserver/syslog-parser.rules ]; then
	cp $CSPCHOME/resources/server/mainserver/syslog-parser.rules $BACKUP_DIR
	echo "INFO: Copied $CSPCHOME/resources/server/mainserver/syslog-parser.rules to backup dir"
fi

if [ -f $CSPCHOME/resources/server/syslogparser/syslogparser.properties ]; then
	cp $CSPCHOME/resources/server/syslogparser/syslogparser.properties $BACKUP_DIR
	echo "INFO: Copied $CSPCHOME/resources/server/syslogparser/syslogparser.properties to backup dir"
fi

if [ -f /etc/scproxyinfo ]; then
	cp /etc/scproxyinfo $BACKUP_DIR
	echo "INFO: Copied /etc/scproxyinfo to backup dir"
fi

if [ -f /opt/ConcsoTgw/tail-end-gateway-decoupled/conf/proxy_conf.properties ]; then
	cp /opt/ConcsoTgw/tail-end-gateway-decoupled/conf/proxy_conf.properties $BACKUP_DIR 
	echo "INFO: Copied /opt/ConcsoTgw/tail-end-gateway-decoupled/conf/proxy_conf.properties to backup dir"
fi

if [ -f $CSPCHOME/resources/kernel/tg/proxyconfig.properties ]; then
	cp $CSPCHOME/resources/kernel/tg/proxyconfig.properties $BACKUP_DIR
	echo "INFO: Copied $CSPCHOME/resources/kernel/tg/proxyconfig.properties to backup dir"
fi
if [ -f /etc/ssh/sshd_config  ]; then
        cp /etc/ssh/sshd_config  $BACKUP_DIR
        echo "INFO: Copied /etc/ssh/sshd_config  backup dir"
fi

if [ -f /etc/passwd ]; then
        cp /etc/passwd $BACKUP_DIR
        echo "INFO: Copied /etc/passwd to backup dir"
fi

if [ -f /etc/shadow ]; then
        cp /etc/shadow $BACKUP_DIR
        echo "INFO: Copied /etc/shadow to backup dir"
fi

if [ -f /etc/group ]; then
        cp /etc/group $BACKUP_DIR
        echo "INFO: Copied /etc/group to backup dir"
fi

if [ -f /etc/rsyslog.conf ]; then
        cp /etc/rsyslog.conf $BACKUP_DIR
        echo "INFO: Copied /etc/rsyslog.conf to backup dir"
fi

if [ -f /etc/syslog-ng/syslog-ng.conf ]; then
	cp /etc/syslog-ng/syslog-ng.conf $BACKUP_DIR
	echo "INFO: Copied /etc/syslog-ng/syslog-ng.conf to backup dir"
fi

cat ~/.bash_history > $BACKUP_DIR/history.txt
echo "INFO: Copied history to backup dir"

route -n > route.txt
if [ -f route.txt ]; then
	while read line; do
	ip=`echo $line | awk '{print $1}'`
	netmask=`echo $line | awk '{print $3}'`
	gateway=`echo $line | awk '{print $2}'`
	iname=`echo $line | awk '{print $8}'`
	if [ $ip != "Kernel" ] && [ $ip != "Destination" ] && [ $ip != "0.0.0.0" ]; then
        	prefix=`ipcalc -p 1.1.1.1 $netmask | sed -n 's/^PREFIX=\(.*\)/\/\1/p'`
	        echo "$ip$prefix via $gateway dev $iname" >> $BACKUP_DIR/route-$iname

	fi
	done < route.txt 
	rm -f route.txt
	echo "INFO: Copied route table entries"
fi

if [ -f files.conf ]; then
	mkdir -p $BACKUP_DIR/cust_files
        while read line; do
	cp -rp $line $BACKUP_DIR/cust_files
        done < files.conf
        echo "INFO: Copied custom files to backup dir"
fi
if [ -f dir.conf ]; then
        mkdir -p $BACKUP_DIR/cust_dir
        while read line; do
        cp -rp $line $BACKUP_DIR/cust_dir
        done < dir.conf
        echo "INFO: Copied custom directories to backup dir"
fi

if [ -f dir.conf ]; then
        cp dir.conf  $INPUT_DIR/cspc28backup
fi
if [ -f files.conf ]; then
        cp files.conf  $INPUT_DIR/cspc28backup
fi



numOfInt=`ifconfig | grep 'Link ' | awk '{print $1}' | grep -v 'lo' | grep -v 'eth0'`
echo $numOfInt > $BACKUP_DIR/numoOfInt.txt

pushd . 2>&1 > /dev/null
cd  $INPUT_DIR
/bin/tar cpf cspc28backup.tar cspc28backup 2>&1 > /dev/null
echo -e "INFO: Backup completed successfully , Please move $INPUT_DIR/cspc28backup.tar and execute 'BackupRestore.sh -r' in 2.10 box"
popd  2>&1 > /dev/null
exit 0;
}

restore() { 
	uid=`id -u`
	if [ "$uid" != "0" ]
	then
        	echo "This need to be run from root user"
	fi
	#CURR_VER=`grep CASP /opt/LCM/info/version.txt`
	#CENTOS_RELEASE=$(cat /etc/centos-release | tr -dc '0-9.'|cut -d. -f1)
	#if [ $CENTOS_RELEASE != "8" ]; then
	#	echo "ERROR : This option is supported only on CentOS 8 box"
	#	exit 1
	#fi


        read -p "Are you sure you want to run the script? This will reboot the box  (y/n)" -n 1 -r
        echo
        if [[ ! $REPLY =~ ^[Yy]$ ]]
        then

                echo -e "\nINFO: Restore terminated"
                exit 1
        fi


	BACKUP_DIR="backup28"
	CURR_BACKUP="currentbk"
	mkdir -p $CURR_BACKUP
	if [ -d $BACKUP_DIR ]; then
		echo "INFO: Backup directory existing , proceeding with applying changes"
		if [ -f $BACKUP_DIR/resolv.conf ]; then
			cp $BACKUP_DIR/resolv.conf /etc/resolv.conf		
			echo "INFO: Restored /etc/resolv.conf"
		fi

		if [ -f /etc/logrotate.conf ]; then
			cp /etc/logrotate.conf $CURR_BACKUP
			cp $BACKUP_DIR/logrotate.conf /etc/logrotate.conf
			echo "INFO: Restored /etc/logrorate.conf"

		fi
		if [ -f $CSPCHOME/nft.rules ]; then
			cp $CSPCHOME/nft.rules $CURR_BACKUP
			iptables-restore-translate -f $BACKUP_DIR/rules.iptables > $CSPCHOME/nft.rules
			nft -f $CSPCHOME/nft.rules 
			echo "INFO: IPtables rules are restored"
		fi 
		if [ -f /var/spool/cron/root ]; then
        		cp /var/spool/cron/root $CURR_BACKUP
			cp $BACKUP_DIR/root /var/spool/cron/root
			echo "INFO: Cron entries restored"	
		fi	
                if [ -f /etc/hosts ]; then
                        cp /etc/hosts $CURR_BACKUP
                        cp $BACKUP_DIR/hosts /etc/hosts
                        echo "INFO: /etc/hosts entries restored"
                fi
		if [ -f $CSPCHOME/webui/tomcat/conf/cspcgxt ]; then
			cp $CSPCHOME/webui/tomcat/conf/cspcgxt $CURR_BACKUP
			cp $BACKUP_DIR/cspcgxt $CSPCHOME/webui/tomcat/conf/cspcgxt
			echo "INFO: Restored $CSPCHOME/webui/tomcat/conf/cspcgxt"  
		fi
		if [ -f $CSPCHOME/resources/server/mainserver/syslog-parser.rules ]; then
			cp $CSPCHOME/resources/server/mainserver/syslog-parser.rules $CURR_BACKUP
			cp $BACKUP_DIR/syslog-parser.rules  $CSPCHOME/resources/server/mainserver/syslog-parser.rules
			echo "INFO: Restored $CSPCHOME/resources/server/mainserver/syslog-parser.rules"
		fi
		if [ -f $CSPCHOME/resources/server/syslogparser/syslogparser.properties ]; then
			cp $CSPCHOME/resources/server/syslogparser/syslogparser.properties $CURR_BACKUP
			cp $BACKUP_DIR/syslogparser.properties $CSPCHOME/resources/server/syslogparser/syslogparser.properties
			echo "INFO: Restored $CSPCHOME/resources/server/syslogparser/syslogparser.properties"
		fi	
		if [ -f /etc/scproxyinfo ] &&  [ -f $BACKUP_DIR/scproxyinfo ]
		then
			cp /etc/scproxyinfo $CURR_BACKUP
			cp $BACKUP_DIR/scproxyinfo /etc/scproxyinfo
   			cp -f /opt/cisco/ss/adminshell/bin/proxyconf.sh /etc/profile.d/
   			chmod -f 644 /etc/scproxyinfo
   			chmod -f 666 /etc/profile.d/proxyconf.sh
			echo "INFO: Restored /etc/scproxyinfo"
		elif [ -f  $BACKUP_DIR/scproxyinfo ]
		then
			cp $BACKUP_DIR/scproxyinfo /etc/scproxyinfo
                        cp -f /opt/cisco/ss/adminshell/bin/proxyconf.sh /etc/profile.d/
                        chmod -f 644 /etc/scproxyinfo
                        chmod -f 666 /etc/profile.d/proxyconf.sh
                        echo "INFO: Restored /etc/scproxyinfo"

		fi
		if [ -f /opt/ConcsoTgw/tail-end-gateway-decoupled/conf/proxy_conf.properties ] && [ -f  $BACKUP_DIR/proxy_conf.properties ]
		then
			cp /opt/ConcsoTgw/tail-end-gateway-decoupled/conf/proxy_conf.properties $CURR_BACKUP
			cp $BACKUP_DIR/proxy_conf.properties /opt/ConcsoTgw/tail-end-gateway-decoupled/conf/proxy_conf.properties
			echo "INFO: Restored /opt/ConcsoTgw/tail-end-gateway-decoupled/conf/proxy_conf.properties"
		elif [ -f  $BACKUP_DIR/proxy_conf.properties ]
		then
			cp $BACKUP_DIR/proxy_conf.properties /opt/ConcsoTgw/tail-end-gateway-decoupled/conf/proxy_conf.properties
                        echo "INFO: Restored /opt/ConcsoTgw/tail-end-gateway-decoupled/conf/proxy_conf.properties"
		fi
	
		if [ -f $CSPCHOME/resources/kernel/tg/proxyconfig.properties ]; then
			cp $CSPCHOME/resources/kernel/tg/proxyconfig.properties $CURR_BACKUP
			cp $BACKUP_DIR/proxyconfig.properties $CSPCHOME/resources/kernel/tg/proxyconfig.properties
			echo "INFO: Restored $CSPCHOME/resources/kernel/tg/proxyconfig.properties"
		else
			cp $BACKUP_DIR//proxyconfig.properties $CSPCHOME/resources/kernel/tg/proxyconfig.properties
			chown casuser:casusers $CSPCHOME/resources/kernel/tg/proxyconfig.properties
			chmod 740 $CSPCHOME/resources/kernel/tg/proxyconfig.properties
                        echo "INFO: Restored $CSPCHOME/resources/kernel/tg/proxyconfig.properties"
	
		fi

		cp $BACKUP_DIR/route* /etc/sysconfig/network-scripts
		chmod 644 /etc/sysconfig/network-scripts/route*

		if [ -f files.conf ]; then
        		while read line; do
        		file_name=`echo $line | awk -F "/" '{print $NF}'`
        		#echo "File name $file_name"
        		cp -rp $BACKUP_DIR/cust_files/$file_name $line
        		done < files.conf
        		echo "INFO: Restored custom files "
		fi

                if [ -f dir.conf ]; then
                        while read line; do
			mkdir -p $line
                        dir_name=`echo $line | awk -F "/" '{print $NF}'`
                        ex_dir_name=`echo $line | awk -F\/ 'BEGIN {OFS="/"} {$NF=""} 1'`
                        #echo "Dir name $file_name"
                        cp -rp $BACKUP_DIR/cust_dir/$dir_name $ex_dir_name 
                        done < dir.conf
                        echo "INFO: Restored custom files "
                fi

		echo -e "\nINFO: All changes has been applied, box is going for reboot now, try login after sometime"
		sleep 10
		reboot
	fi
	echo "Restore is completed"
	exit 0
}


if [ $# -eq 0 ]
then
usage
fi

while true ; do
	case "$1" in
                -h|--help) usage ;;
                -b|--backup) backup ;;
		-r|--restore) restore ;;

	*) echo "[ERROR] : Internal error!" ; exit 1 ;;
    esac
done

